#ifndef PCIE40_B2CONFIG_H
#define PCIE40_B2CONFIG_H
// Header file for B2 configuration functions for PCIe40 
#include <string>

// Constants
// BAR number
#define SLC_BAR 2
#define SIZE_MEMO_FPGA 1024

// Functions to read/write registers and stream files to Front End
#ifndef USE_PCIE40
extern "C" {
#endif 
  /// returns the FPGA firmware's version number
  std::string pcie40_fpgaVersion(int dev);
  /// Disable one TX link (physically via the minipod configuration)
  /// void pcie40_disableMinipodLink( int dev , int ch ) ;
  /// Enable one TX link (physically via the minipod configuration)
  /// void pcie40_enableMinipodLink( int dev , int ch ) ; 
  /// Reset one optical link via minipod
  void pcie40_resetMinipodLink( int dev , int ch ) ;
  /// Gives the status (on = 1, off = 0) of the minipod link
  int pcie40_statusMinipodLink( int dev , int ch ) ;
  /// Configure the PLLs for belle2 clock usage
  int pcie40_configurePLLs( int dev ) ; 
  /// Frequency of the FTSW input clock 
  int pcie40_inputPLLFrequency( int dev ) ; 
  /// Lock Status of PLL (locked = 1 , not locked = 0 ), for the 3 external PLLs 
  /// ( 1 = PLL for input channels 0 to 23, 2 = PLL for input channels 24 to 47, 3 = FTSW PLL)
  int pcie40_pllLockStatus( int dev , int pll ) ;
  /// Input status of PLL (good = 1 , loss of signal = 0) 
  int pcie40_pllInputStatus( int dev , int pll ) ;
  /// Frequency status of PLL (good = 1 , out of frequency = 0 ) 
  int pcie40_pllFrequencyStatus( int dev , int pll ) ;
  /// Loss of Lock counter (1 if no LOL detected, 0 otherwise) 
  int pcie40_pllLockCounter( int dev , int pll ) ;
  /// Loss of Signal counter (1 if no LOS detected, 0 otherwise)
  int pcie40_pllInputCounter( int dev , int pll ) ; 
  /// Out of Frequency counter (1 if no OOF detected, 0 otherwise) 
  int pcie40_pllFrequencyCounter( int dev , int pll ) ;
  /// Reset of Loss of Lock counter
  void pcie40_pllLockReset( int dev , int pll ) ;
  /// Reset of Loss of Signal counter
  void pcie40_pllInputReset( int dev , int pll ) ; 
  /// Reset of Out of Frequency counter
  void pcie40_pllFrequencyReset( int dev , int pll ) ;

#ifndef USE_PCIE40
}
#endif

#endif // PCIE40_B2CONFIG_H
