#ifndef ECS_H
#define ECS_H

#include <linux/ioctl.h>

typedef struct {
	int size;
} ecs_arg_t;

#define    ECS_IOCTL_SET_SIZE   _IOW('e',0, ecs_arg_t *) 
#define    ECS_IOCTL_GET_SIZE   _IOR('e',1, ecs_arg_t *)

#ifdef __cplusplus
extern "C" {
#endif

void ecs_printSetup();

int  ecs_open(int dev, int bar);
void ecs_close(int dev, int bar);

int      ecs_write(int dev, int bar, unsigned add, int val);
unsigned ecs_read(int dev, int bar, unsigned add);
int 	 ecs_writeBloc(int dev, int bar, unsigned add, int size, unsigned *val);
int 	 ecs_readBloc(int dev, int bar, int unsigned, int size, unsigned *val);
int      dma_launch( int dev , unsigned int ** data ) ;

//Adaptation with old library
int ecs_iowrBar(int dev, int bar, unsigned add, unsigned *val);
int ecs_iordBar(int dev, int bar, unsigned add, unsigned *val);
int ecs_iowrBarBloc(int dev, int bar, unsigned add, int size, unsigned *val);
int ecs_iordBarBloc(int dev, int bar, unsigned add, int size, unsigned *val);

// functions to use for user space registers R/W
int  ecs_openUser(int dev);
void ecs_closeUser(int dev);
int  ecs_iordUser(int dev, unsigned add, unsigned *val);
int  ecs_iowrUser(int dev, unsigned add, unsigned *val);
int  ecs_iordBlocUser(int dev, unsigned add, int size, unsigned *val);
int  ecs_iowrBlocUser(int dev, unsigned add, int size, unsigned *val);

// functions to use to access the LLI ressources
int ecs_openLli(int dev);
void ecs_closeLli(int dev);
int  ecs_iordLli(int dev, unsigned add, unsigned *val);
int  ecs_iowrLli(int dev, unsigned add, unsigned *val);
int  ecs_iordBlocLli(int dev, unsigned add, int size, unsigned *val);
int  ecs_iowrBlocLli(int dev, unsigned add, int size, unsigned *val);

// functions to use to access the I2C core registers
int  ecs_iordI2c(int dev, unsigned add, unsigned *val);
int  ecs_iowrI2c(int dev, unsigned add, unsigned *val);
int  ecs_iordI2c_slow(int dev, unsigned add, unsigned *val);
int  ecs_iowrI2c_slow(int dev, unsigned add, unsigned *val);

// function to access the PHY registers space
int ecs_iordPhy(int dev, unsigned add, unsigned *val);
int ecs_iowrPhy(int dev, unsigned add, unsigned *val);

#ifdef __cplusplus
} //extern "C"
#endif

#endif
